--[[ ---------------------------------------------------------------------------

DebuffEnough: Debuff monitor

DebuffEnough is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DebuffEnough is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with DebuffEnough.	If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]

--[[ ---------------------------------------------------------------------------
	 Addon and libraries
----------------------------------------------------------------------------- ]]

local L = LibStub("AceLocale-3.0"):GetLocale("DebuffEnough")
local C = LibStub("AceConfigDialog-3.0")


--[[ ---------------------------------------------------------------------------
	 Create dropdowns
----------------------------------------------------------------------------- ]]

local logLevelsDropdown = {}

if DebuffEnough.logLevels then
	for logname,id in pairs(DebuffEnough.logLevels) do
		logLevelsDropdown[id] = L[logname]
	end
end


--[[ ---------------------------------------------------------------------------
	 Config options
----------------------------------------------------------------------------- ]]
DebuffEnough.options = {
	name = "DebuffEnough",
	handler = DebuffEnough,
	type = "group",
	args = {
		general = {
			type = "group",
			name = L["General"],
			order = 0,
			set = "SetProfileParam",
			get = "GetProfileParam",
			cmdHidden = true,
			args = {
				enable = {
					type = "toggle",
					name = L["Enable"],
					desc = L["Enable or disable the addon"],
					width = "full",
					order = 0,
					set = function(info, v)
						DebuffEnough:SetProfileParam("enable", v)
						if v then
							DebuffEnough:DoEnable()
						else
							DebuffEnough:DoDisable()
						end
				  	end	
				},
				solo = {
					type = "toggle",
					name = L["Enable While Solo"],
					desc = L["Enable DebuffEnough while not in a party/raid"],
					width = "full",
					order = 1,
					set = function(info, v)
							DebuffEnough:SetProfileParam("solo", v)
							DebuffEnough:CheckRaidChange()
				  		end	
				},
				fade = {
					type = "range",
					name = L["Fade"],
					desc = L["Fade frame after this many seconds of being debuff enough. Set to 0 for no fading"],
					min = 0,
					max = 60,
					step = 1,
					bigStep = 1,
					width = "full",
					order = 3,
					set = function(info, v)
							DebuffEnough:SetProfileParam("fade", v)
							DebuffEnough:UpdateDisplay()
				  		end	
				},
				logLevel = {
					type = "select",
					name = L["Log Level"],
					desc = L["Determines the amount of output from the addon"],
					order = 99,
					values = logLevelsDropdown,
					disabled = function() return not LibStub("LibLogger-1.0", true) end,
				},
			},
		},
		display = {
			type = "group",
			name = L["Display"],
			cmdHidden = true,
			order = 1,
			set = "SetProfileParam",
			get = "GetProfileParam",
			args = {
				show = {
					type = "toggle",
					name = L["Show Display"],
					desc = L["Shows the display"],
					order = 0,
					width = "full",
					set = function(info, v)
							DebuffEnough:SetProfileParam("show", v)
							DebuffEnough:UpdateVisible()
				  		end	
				},
				lock = {
					type = "toggle",
					name = L["Lock"],
					desc = L["Lock the display"],
					order = 1,
					width = "full",
					set = function(info, v)
					  		DebuffEnough:SetProfileParam("lock", v)
					  		DebuffEnough:UpdateDisplay()
				  		end	
				},
				verbose = {
					type = "toggle",
					name = L["Verbose"],
					desc = L["Verbose mode displays a readout frame next to the indicator that shows you all of the information that would normally be shown in the mouseover tooltip."],
					order = 2,
					width = "full",
					set = function(info, v)
					  		DebuffEnough:SetProfileParam("verbose", v)
					  		DebuffEnough:UpdateDisplay()
				  		end	
				},
				strata = {
					type = "select",
					name = L["Strata"],
					desc = L["Strata of the display"],
					order = 4,
					values = {HIGH = _G["HIGH"], MEDIUM = _G["AUCTION_TIME_LEFT2"], LOW = _G["LOW"], BACKGROUND = _G["BACKGROUND"]},
					set = function(info, v)
				 			DebuffEnough:SetProfileParam("strata", v)
				  			DebuffEnough:UpdateDisplay()
						end	
				},
				indicatorsettings = {
					type = "group",
					name = L["Indicator"],
					set = "SetProfileParam",
					get = "GetProfileParam",
					guiInline = true,
					order = 5,
					args = {
						indscale = {
							type = "range",
							name = L["Scale"],
							desc = L["Scale of the indicator"],
							min = 1,
							max = 150,
							step = 1,
							bigStep = 1,
							order = 1,
							set = function(info, v)
									DebuffEnough:SetProfileParam("indscale", v)
			  						DebuffEnough:UpdateDisplay()
							 	  end						
						},
						indalpha = {
							type = "range",
							name = L["Opacity"],
							desc = L["Opacity of the indicator"],
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.05,
							order = 2,
							set = function(info, v)
				 					DebuffEnough:SetProfileParam("indalpha", v)
				  					DebuffEnough:UpdateDisplay()
							 	  end	
						},
						inddebuffcolor = {
							type = "color",
							name = L["Debuff Color"],
							desc = L["Color to display when your target is debuff enough"],
							hasAlpha = false,
							width = "full",
							order = 3,
							get = function(info) return unpack(DebuffEnough:GetProfileParam("inddebuffcolor")) end,
							set = function(info, r, g, b)
									DebuffEnough:SetProfileParam("inddebuffcolor", {r, g, b})
									DebuffEnough:UpdateDisplay()
								  end
						},
						indundebuffcolor = {
							type = "color",
							name = L["Undebuff Color"],
							desc = L["Color to display when your target is not debuff enough"],
							hasAlpha = false,
							width = "full",
							order = 4,
							get = function(info) return unpack(DebuffEnough:GetProfileParam("indundebuffcolor")) end,
							set = function(info, r, g, b)
									DebuffEnough:SetProfileParam("indundebuffcolor", {r, g, b})
									DebuffEnough:UpdateDisplay()
								  end
						},
						indwarndebuffcolor = {
							type = "color",
							name = L["Warn Debuff Color"],
							desc = L["Color to display when the optimal debuff for a slot is not being applied"],
							hasAlpha = false,
							width = "full",
							order = 5,
							get = function(info) return unpack(DebuffEnough:GetProfileParam("indwarndebuffcolor")) end,
							set = function(info, r, g, b)
									DebuffEnough:SetProfileParam("indwarndebuffcolor", {r, g, b})
									DebuffEnough:UpdateDisplay()
								  end
						},
						indbordercolor = {
							type = "color",
							name = L["Border Color"],
							desc = L["Color of the indicator border"],
							hasAlpha = false,
							width = "full",
							order = 6,
							get = function(info) return unpack(DebuffEnough:GetProfileParam("indbordercolor")) end,
							set = function(info, r, g, b)
									DebuffEnough:SetProfileParam("indbordercolor", {r, g, b})
									DebuffEnough:UpdateDisplay()
								  end
						},
						indbordersize = {
							type = "range",
							name = L["Border Size"],
							desc = L["Thickness of the indicator border"],
							min = 1,
							max = 64,
							step = 1,
							bigStep = 1,
							order = 7,
							set = function(info, v)
									DebuffEnough:SetProfileParam("indbordersize", v)
			  						DebuffEnough:UpdateDisplay()
								  end						
						},
						indbginset = {
							type = "range",
							name = L["Background Inset"],
							desc = L["How far inside the border to set the background of the indicator"],
							min = 1,
							max = 64,
							step = 1,
							bigStep = 1,
							order = 10,
							set = function(info, v)
									DebuffEnough:SetProfileParam("indbginset", v)
			  						DebuffEnough:UpdateDisplay()
								  end						
						},
						indbgtexture = {
		  					type = "select",
		  					dialogControl = "LSM30_Background",
		  					name = L["Background Texture"],
		  					desc = L["The background texture of the indicator"],
		  					order = 11,
		  					values = AceGUIWidgetLSMlists.background,
		  					set = function(info, v)
				 					DebuffEnough:SetProfileParam("indbgtexture", v)
				  					DebuffEnough:UpdateDisplay()
							  	  end	
	       				},
	       				indbordertexture = {
		  					type = "select",
		  					dialogControl = "LSM30_Border",
		  					name = L["Border Texture"],
		  					desc = L["The border texture of the indicator"],
		  					order = 12,
		  					values = AceGUIWidgetLSMlists.border, 
		  					set = function(info, v)
				 					DebuffEnough:SetProfileParam("indbordertexture", v)
				  					DebuffEnough:UpdateDisplay()
							end	
	       				},
					},
				},
				readoutsettings = {
					type = "group",
					name = L["Readout"],
					set = "SetProfileParam",
					get = "GetProfileParam",
					guiInline = true,
					order = 5,
					args = {
						readanchorpoint = {
							type = "select",
							name = L["Anchor Point"],
							desc = L["The part of the readout that should be anchored to the indicator"],
							disabled = function() return not DebuffEnough:GetProfileParam("verbose") end,
							order = 1,
							values = {["CENTER"] = L["CENTER"], ["TOP"] = L["TOP"], ["BOTTOM"] = L["BOTTOM"], ["LEFT"] = L["LEFT"], ["RIGHT"] = L["RIGHT"], ["TOPLEFT"] = L["TOPLEFT"], ["TOPRIGHT"] = L["TOPRIGHT"], ["BOTTOMLEFT"] = L["BOTTOMLEFT"], ["BOTTOMRIGHT"] = L["BOTTOMRIGHT"]},
							set = function(info, v)
									DebuffEnough:SetProfileParam("readanchorpoint", v)
			  						DebuffEnough:UpdateDisplay()
							 	  end
						},
						readanchor = {
							type = "select",
							name = L["Anchor"],
							desc = L["Where on the indicator the readout should be anchored"],
							disabled = function() return not DebuffEnough:GetProfileParam("verbose") end,
							order = 2,
							values = {["CENTER"] = L["CENTER"], ["TOP"] = L["TOP"], ["BOTTOM"] = L["BOTTOM"], ["LEFT"] = L["LEFT"], ["RIGHT"] = L["RIGHT"], ["TOPLEFT"] = L["TOPLEFT"], ["TOPRIGHT"] = L["TOPRIGHT"], ["BOTTOMLEFT"] = L["BOTTOMLEFT"], ["BOTTOMRIGHT"] = L["BOTTOMRIGHT"]},
							set = function(info, v)
									DebuffEnough:SetProfileParam("readanchor", v)
			  						DebuffEnough:UpdateDisplay()
							 	  end
						},
						readscale = {
							type = "range",
							name = L["Scale"],
							desc = L["Scale of the readout"],
							disabled = function() return not DebuffEnough:GetProfileParam("verbose") end,
							min = 1,
							max = 150,
							step = 1,
							bigStep = 1,
							order = 3,
							set = function(info, v)
									DebuffEnough:SetProfileParam("readscale", v)
			  						DebuffEnough:UpdateDisplay()
							 	  end						
						},
						readalpha = {
							type = "range",
							name = L["Opacity"],
							desc = L["Opacity of the readout"],
							disabled = function() return not DebuffEnough:GetProfileParam("verbose") end,
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.05,
							order = 4,
							set = function(info, v)
				 					DebuffEnough:SetProfileParam("readalpha", v)
				  					DebuffEnough:UpdateDisplay()
							 	  end	
						},
						readbgcolor = {
							type = "color",
							name = L["Background Color"],
							desc = L["Color of the readout background"],
							disabled = function() return not DebuffEnough:GetProfileParam("verbose") end,
							hasAlpha = false,
							width = "full",
							order = 5,
							get = function(info) return unpack(DebuffEnough:GetProfileParam("readbgcolor")) end,
							set = function(info, r, g, b)
									DebuffEnough:SetProfileParam("readbgcolor", {r, g, b})
									DebuffEnough:UpdateDisplay()
								  end
						},
						readbordercolor = {
							type = "color",
							name = L["Border Color"],
							desc = L["Color of the readout border"],
							disabled = function() return not DebuffEnough:GetProfileParam("verbose") end,
							hasAlpha = false,
							width = "full",
							order = 6,
							get = function(info) return unpack(DebuffEnough:GetProfileParam("readbordercolor")) end,
							set = function(info, r, g, b)
									DebuffEnough:SetProfileParam("readbordercolor", {r, g, b})
									DebuffEnough:UpdateDisplay()
								  end
						},
						readbordersize = {
							type = "range",
							name = L["Border Size"],
							desc = L["Thickness of the readout border"],
							min = 1,
							max = 64,
							step = 1,
							bigStep = 1,
							order = 7,
							set = function(info, v)
									DebuffEnough:SetProfileParam("readbordersize", v)
			  						DebuffEnough:UpdateDisplay()
								  end						
						},
						readbginset = {
							type = "range",
							name = L["Background Inset"],
							desc = L["How far inside the border to set the background of the readout"],
							min = 1,
							max = 64,
							step = 1,
							bigStep = 1,
							order = 10,
							set = function(info, v)
									DebuffEnough:SetProfileParam("readbginset", v)
			  						DebuffEnough:UpdateDisplay()
								  end						
						},
						readbgtexture = {
		  					type = "select",
		  					dialogControl = "LSM30_Background",
		  					name = L["Background Texture"],
		  					desc = L["The background texture of the readout"],
		  					order = 11,
		  					values = AceGUIWidgetLSMlists.background,
		  					set = function(info, v)
				 					DebuffEnough:SetProfileParam("readbgtexture", v)
				  					DebuffEnough:UpdateDisplay()
							  	  end	
	       				},
	       				readbordertexture = {
		  					type = "select",
		  					dialogControl = "LSM30_Border",
		  					name = L["Border Texture"],
		  					desc = L["The border texture of the readout"],
		  					order = 12,
		  					values = AceGUIWidgetLSMlists.border, 
		  					set = function(info, v)
				 					DebuffEnough:SetProfileParam("readbordertexture", v)
				  					DebuffEnough:UpdateDisplay()
							end	
	       				},
	       				readfont = {
		  					type = "select",
		  					dialogControl = "LSM30_Font",
		  					name = L["Font"],
		  					desc = L["Font used in the readout"],
		  					order = 13,
		  					values = AceGUIWidgetLSMlists.font,
		  					set = function(info, v)
				 					DebuffEnough:SetProfileParam("readfont", v)
				  					DebuffEnough:UpdateDisplay()
							end	
	       				},
	       				readfontsize = { 
		  					type = "range",
		  					name = L["Font Size"],
		  					desc = L["Size of the font used in the readout"],
		  					min = 1,
							max = 30,
							step = 1,
							bigStep = 1,
							order = 14,
							set = function(info, v)
				 					DebuffEnough:SetProfileParam("readfontsize", v)
				  					DebuffEnough:UpdateDisplay()
							end	
				        },
					},
				},
			},
		},
		debuffs = {
			type = "group",
			name = L["Debuffs"],
			order = 2,
			set = "SetProfileParam",
			get = "GetProfileParam",
			cmdHidden = true,
			args = {
				showbest = {
					type = "toggle",
					name = L["Show Best Debuff Only"],
					desc = L["Only show the best debuff for the given slot. If unchecked, all possible debuffs are shown, and the best options are highlighted."],
					width = "full",
					order = 1,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showbest", v)
						DebuffEnough:RenderTooltip()
				  	end	,
				},
				showclass= {
					type = "toggle",
					name = L["Show My Class Debuffs Only"],
					desc = L["Only show debuffs that your class can cast"],
					width = "full",
					order = 2,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showclass", v)
						DebuffEnough:GetAvailableDebuffs()
				  	end	,
				},
				showhealingtaken = {
					type = "toggle",
					name = L["Show Healing Taken Debuffs"],
					desc = L["Show debuffs that reduce healing taken"],
					width = "full",
					order = 3,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showhealingtaken", v)
						DebuffEnough:GetAvailableDebuffs()
				  	end	,
				},
				showcastspeed= {
					type = "toggle",
					name = L["Show Cast Speed Debuffs"],
					desc = L["Show debuffs that reduce cast speed"],
					width = "full",
					order = 4,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showcastspeed", v)
						DebuffEnough:GetAvailableDebuffs()
				  	end	,
				},
			},
		},
	},
}

DebuffEnough.options.args[L["config"]] = {
	type = "execute",
	name = L["config"],
	desc = L["Toggle the Configuration Dialog"],
	func = "ToggleConfigDialog",
	guiHidden = true,
	order = 1,
}


--[[ ---------------------------------------------------------------------------
	 Config defaults
----------------------------------------------------------------------------- ]]
DebuffEnough.defaults = {
	profile = {
		enable = true,
		solo = true,
		fade = 0,
		logLevel = 0,
		show = true,
		lock = false,
		verbose = true,
		strata = "BACKGROUND",
		indalpha = 1,
		indscale = 100,
		inddebuffcolor = {0, .75, 0},
		indundebuffcolor = {.75, 0, 0},
		indwarndebuffcolor = {.75, .75, 0},
		indbordercolor = {0, 0, 0},
		indbordersize = 6,
		indbginset = 1,
		indbgtexture = "Blizzard Tooltip",
		indbordertexture = "Blizzard Tooltip",
		readanchorpoint = "TOPLEFT",
		readanchor = "TOPRIGHT",
		readalpha = 1,
		readscale = 100,
		readbgcolor = {0, .75, 0},
		readbordercolor = {0, 0, 0},
		readbordersize = 6,
		readbginset = 1,
		readbgtexture = "Blizzard Tooltip",
		readbordertexture = "Blizzard Tooltip",
		readfont = "Friz Quadrata TT",
		readfontsize = "10",
		showmissingtalents = false,
		showbest = true,
		showclass = false,
		showhealingtaken = false,
		showcastspeed = false,
	},
}


--[[ ---------------------------------------------------------------------------
	 Toggle the config window
----------------------------------------------------------------------------- ]]
function DebuffEnough:ToggleConfigDialog()

   local frame = C.OpenFrames["DebuffEnough"]

   if frame then
	  C:Close("DebuffEnough")
   else
	  C:Open("DebuffEnough")
	  self:SetStatusText(format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set the text displayed at the bottom of the config window
----------------------------------------------------------------------------- ]]
function DebuffEnough:SetStatusText(text)

   local frame = C.OpenFrames["DebuffEnough"]

   if frame then
	  frame:SetStatusText(text)
   end

end


--[[ ---------------------------------------------------------------------------
	 Update config display on a profile change
----------------------------------------------------------------------------- ]]
function DebuffEnough:OnProfileChanged(event, newdb)

   if event ~= "OnProfileDeleted" then
	  if LibStub("LibLogger-1.0", true) then self:SetLogLevel(self.db.profile.logLevel) end
	  self:SetStatusText(format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set a saved variable config option
----------------------------------------------------------------------------- ]]
function DebuffEnough:SetProfileParam(var, value)

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.trace then self:trace("Setting parameter %s to %s.", varName, tostring(value)) end

   self.db.profile[varName] = value

   if varName == "logLevel" and LibStub("LibLogger-1.0", true) then
	  self:SetLogLevel(value)
   end

end


--[[ ---------------------------------------------------------------------------
	 Get a saved variable config option
----------------------------------------------------------------------------- ]]
function DebuffEnough:GetProfileParam(var) 

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.spam then self:spam("Getting parameter %s as %s", varName, tostring(self.db.profile[varName])) end

   return self.db.profile[varName]

end